@@ -1894,7 +1894,7 @@ char **mutt_envlist (void)
  * It's broken out because some other parts of mutt (filter.c) need
  * to set/overwrite environment variables in envlist before execing.
  */
-void mutt_envlist_set (const char *name, const char *value)
+void mutt_envlist_set (const char *name, const char *value, int overwrite)
 {
   char **envp = envlist;
   char work[LONG_STRING];
@@ -1907,7 +1907,11 @@ void mutt_envlist_set (const char *name, const char *value)
   while (envp && *envp)
   {
     if (!mutt_strncmp (name, *envp, len) && (*envp)[len] == '=')
+    {
+      if (!overwrite)
+        return;
       break;
+    }
     envp++;
     count++;
   }
@@ -2022,7 +2026,7 @@ static int parse_setenv(BUFFER *tmp, BUFFER *s, unsigned long data, BUFFER *err)
 
   name = safe_strdup (tmp->data);
   mutt_extract_token (tmp, s, 0);
-  mutt_envlist_set (name, tmp->data);
+  mutt_envlist_set (name, tmp->data, 1);
   FREE (&name);
 
   return 0;
