@@ -783,6 +783,7 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
   int alladdr = 0;
   int or = 0;
   int implicit = 1;	/* used to detect logical AND operator */
+  int isalias = 0;
   const struct pattern_flags *entry;
   char *p;
   char *buf;
@@ -805,6 +806,10 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	ps.dptr++;
 	not = !not;
 	break;
+      case '@':
+	ps.dptr++;
+	isalias = !isalias;
+	break;
       case '|':
 	if (!or)
 	{
@@ -830,6 +835,7 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	implicit = 0;
 	not = 0;
 	alladdr = 0;
+	isalias = 0;
 	break;
       case '%':
       case '=':
@@ -859,8 +865,10 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	  last = tmp;
 	  tmp->not ^= not;
 	  tmp->alladdr |= alladdr;
+	  tmp->isalias |= isalias;
 	  not = 0;
 	  alladdr = 0;
+	  isalias = 0;
 	  /* compile the sub-expression */
 	  buf = mutt_substrdup (ps.dptr + 1, p);
 	  if ((tmp2 = mutt_pattern_comp (buf, flags, err)) == NULL)
@@ -888,10 +896,12 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	tmp = new_pattern ();
 	tmp->not = not;
 	tmp->alladdr = alladdr;
+	tmp->isalias = isalias;
         tmp->stringmatch = (*ps.dptr == '=') ? 1 : 0;
         tmp->groupmatch  = (*ps.dptr == '%') ? 1 : 0;
 	not = 0;
 	alladdr = 0;
+	isalias = 0;
 
 	if (last)
 	  last->next = tmp;
@@ -957,8 +967,10 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	last = tmp;
 	tmp->not ^= not;
 	tmp->alladdr |= alladdr;
+	tmp->isalias |= isalias;
 	not = 0;
 	alladdr = 0;
+	isalias = 0;
 	ps.dptr = p + 1; /* restore location */
 	break;
       default:
@@ -983,19 +995,19 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 }
 
 static int
-perform_and (pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx, HEADER *hdr)
+perform_and (pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx, HEADER *hdr, pattern_cache_t *cache)
 {
   for (; pat; pat = pat->next)
-    if (mutt_pattern_exec (pat, flags, ctx, hdr) <= 0)
+    if (mutt_pattern_exec (pat, flags, ctx, hdr, cache) <= 0)
       return 0;
   return 1;
 }
 
 static int
-perform_or (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx, HEADER *hdr)
+perform_or (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx, HEADER *hdr, pattern_cache_t *cache)
 {
   for (; pat; pat = pat->next)
-    if (mutt_pattern_exec (pat, flags, ctx, hdr) > 0)
+    if (mutt_pattern_exec (pat, flags, ctx, hdr, cache) > 0)
       return 1;
   return 0;
 }
@@ -1010,8 +1022,10 @@ static int match_adrlist (pattern_t *pat, int match_personal, int n, ...)
   {
     for (a = va_arg (ap, ADDRESS *) ; a ; a = a->next)
     {
-      if (pat->alladdr ^ ((a->mailbox && patmatch (pat, a->mailbox) == 0) ||
-	   (match_personal && a->personal && patmatch (pat, a->personal) == 0)))
+      if (pat->alladdr ^
+          ((!pat->isalias || alias_reverse_lookup (a)) &&
+           ((a->mailbox && !patmatch (pat, a->mailbox)) ||
+	    (match_personal && a->personal && !patmatch (pat, a->personal) ))))
       {
 	va_end (ap);
 	return (! pat->alladdr); /* Found match, or non-match if alladdr */
@@ -1080,7 +1094,7 @@ static int match_threadcomplete(struct pattern_t *pat, pattern_exec_flag flags,
     return 0;
   h = t->message;
   if(h)
-    if(mutt_pattern_exec(pat, flags, ctx, h))
+    if(mutt_pattern_exec(pat, flags, ctx, h, NULL))
       return 1;
 
   if(up && (a=match_threadcomplete(pat, flags, ctx, t->parent,1,1,1,0)))
@@ -1094,17 +1108,44 @@ static int match_threadcomplete(struct pattern_t *pat, pattern_exec_flag flags,
   return 0;
 }
 
-/* flags
-   	MUTT_MATCH_FULL_ADDRESS	match both personal and machine address */
-int
-mutt_pattern_exec (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx, HEADER *h)
+
+/* Sets a value in the pattern_cache_t cache entry.
+ * Normalizes the "true" value to 2. */
+static void set_pattern_cache_value (int *cache_entry, int value)
 {
+  *cache_entry = value ? 2 : 1;
+}
+
+/* Returns 1 if the cache value is set and has a true value.
+ * 0 otherwise (even if unset!) */
+static int get_pattern_cache_value (int cache_entry)
+{
+  return cache_entry == 2;
+}
+
+static int is_pattern_cache_set (int cache_entry)
+{
+  return cache_entry != 0;
+}
+
+
+/*
+ * flags: MUTT_MATCH_FULL_ADDRESS - match both personal and machine address
+ * cache: For repeated matches against the same HEADER, passing in non-NULL will
+ *        store some of the cacheable pattern matches in this structure. */
+int
+mutt_pattern_exec (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx, HEADER *h,
+                   pattern_cache_t *cache)
+{
+  int result;
+  int *cache_entry;
+
   switch (pat->op)
   {
     case MUTT_AND:
-      return (pat->not ^ (perform_and (pat->child, flags, ctx, h) > 0));
+      return (pat->not ^ (perform_and (pat->child, flags, ctx, h, cache) > 0));
     case MUTT_OR:
-      return (pat->not ^ (perform_or (pat->child, flags, ctx, h) > 0));
+      return (pat->not ^ (perform_or (pat->child, flags, ctx, h, cache) > 0));
     case MUTT_THREAD:
       return (pat->not ^ match_threadcomplete(pat->child, flags, ctx, h->thread, 1, 1, 1, 1));
     case MUTT_ALL:
@@ -1184,13 +1225,53 @@ mutt_pattern_exec (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx,
            return (pat->not ^ match_adrlist (pat, flags & MUTT_MATCH_FULL_ADDRESS,
                                              2, h->env->to, h->env->cc));
     case MUTT_LIST:	/* known list, subscribed or not */
-      return (pat->not ^ mutt_is_list_cc (pat->alladdr, h->env->to, h->env->cc));
+      if (cache)
+      {
+        cache_entry = pat->alladdr ? &cache->list_all : &cache->list_one;
+        if (!is_pattern_cache_set (*cache_entry))
+          set_pattern_cache_value (cache_entry,
+                                   mutt_is_list_cc (pat->alladdr, h->env->to, h->env->cc));
+        result = get_pattern_cache_value (*cache_entry);
+      }
+      else
+        result = mutt_is_list_cc (pat->alladdr, h->env->to, h->env->cc);
+      return (pat->not ^ result);
     case MUTT_SUBSCRIBED_LIST:
-      return (pat->not ^ mutt_is_list_recipient (pat->alladdr, h->env->to, h->env->cc));
+      if (cache)
+      {
+        cache_entry = pat->alladdr ? &cache->sub_all : &cache->sub_one;
+        if (!is_pattern_cache_set (*cache_entry))
+          set_pattern_cache_value (cache_entry,
+                                   mutt_is_list_recipient (pat->alladdr, h->env->to, h->env->cc));
+        result = get_pattern_cache_value (*cache_entry);
+      }
+      else
+        result = mutt_is_list_recipient (pat->alladdr, h->env->to, h->env->cc);
+      return (pat->not ^ result);
     case MUTT_PERSONAL_RECIP:
-      return (pat->not ^ match_user (pat->alladdr, h->env->to, h->env->cc));
+      if (cache)
+      {
+        cache_entry = pat->alladdr ? &cache->pers_recip_all : &cache->pers_recip_one;
+        if (!is_pattern_cache_set (*cache_entry))
+          set_pattern_cache_value (cache_entry,
+                                   match_user (pat->alladdr, h->env->to, h->env->cc));
+        result = get_pattern_cache_value (*cache_entry);
+      }
+      else
+        result = match_user (pat->alladdr, h->env->to, h->env->cc);
+      return (pat->not ^ result);
     case MUTT_PERSONAL_FROM:
-      return (pat->not ^ match_user (pat->alladdr, h->env->from, NULL));
+      if (cache)
+      {
+        cache_entry = pat->alladdr ? &cache->pers_from_all : &cache->pers_from_one;
+        if (!is_pattern_cache_set (*cache_entry))
+          set_pattern_cache_value (cache_entry,
+                                   match_user (pat->alladdr, h->env->from, NULL));
+        result = get_pattern_cache_value (*cache_entry);
+      }
+      else
+        result = match_user (pat->alladdr, h->env->from, NULL);
+      return (pat->not ^ result);
     case MUTT_COLLAPSED:
       return (pat->not ^ (h->collapsed && h->num_hidden > 1));
    case MUTT_CRYPT_SIGN:
@@ -1350,7 +1431,7 @@ int mutt_pattern_func (int op, char *prompt)
       Context->hdrs[i]->limited = 0;
       Context->hdrs[i]->collapsed = 0;
       Context->hdrs[i]->num_hidden = 0;
-      if (mutt_pattern_exec (pat, MUTT_MATCH_FULL_ADDRESS, Context, Context->hdrs[i]))
+      if (mutt_pattern_exec (pat, MUTT_MATCH_FULL_ADDRESS, Context, Context->hdrs[i], NULL))
       {
 	Context->hdrs[i]->virtual = Context->vcount;
 	Context->hdrs[i]->limited = 1;
@@ -1366,7 +1447,7 @@ int mutt_pattern_func (int op, char *prompt)
     for (i = 0; i < Context->vcount; i++)
     {
       mutt_progress_update (&progress, i, -1);
-      if (mutt_pattern_exec (pat, MUTT_MATCH_FULL_ADDRESS, Context, Context->hdrs[Context->v2r[i]]))
+      if (mutt_pattern_exec (pat, MUTT_MATCH_FULL_ADDRESS, Context, Context->hdrs[Context->v2r[i]], NULL))
       {
 	switch (op)
 	{
@@ -1526,7 +1607,7 @@ int mutt_search_command (int cur, int op)
     {
       /* remember that we've already searched this message */
       h->searched = 1;
-      if ((h->matched = (mutt_pattern_exec (SearchPattern, MUTT_MATCH_FULL_ADDRESS, Context, h) > 0)))
+      if ((h->matched = (mutt_pattern_exec (SearchPattern, MUTT_MATCH_FULL_ADDRESS, Context, h, NULL) > 0)))
       {
 	mutt_clear_error();
 	if (msg && *msg)
